/*
 * (c) Copyright 2000-2011 spring saarlouis
 * All Rights Reserved.
 * 
 * contact info@spring.de for more information
 */
#import <WebKit/WebKit.h>

/**
 * Static tags of the application status
 */
extern NSString *const SPRING_VAR_ACTION;
extern NSString *const SPRING_APP_STARTED;
extern NSString *const SPRING_APP_FOREGROUND;
extern NSString *const SPRING_APP_BACKGROUND;
extern NSString *const SPRING_APP_CLOSED;

/**
 * The notification name for the debugging purpose, register this notification into the notification center by following API or similar:
 * (void)addObserver:(id)observer selector:(SEL)aSelector name:KMA_TAGGING_DEBUGINTERFACE_NOTIFICATION object:(nullable id)anObject;
 * Also, you would need to implement the selecotor method to consume the debug info.
 * The debug info is attached in the notification as a NSDictionary, with keys "Request" and "Statuscode". The info can be fetched by The API: [NSNotification object]
 */
extern NSString *const KMA_TAGGING_DEBUGINTERFACE_NOTIFICATION;

@interface KMA_Spring : NSObject {
}

/**
 *
 */
@property (strong,nonatomic) WKWebView *wkWebView;

/**
 * Enable or disable usage tracking. (default: true)
 */
@property(nonatomic, getter=isTracking) BOOL tracking;
    
/**
 * When set to true (default:false) the library logs the internal actions.
 * Each error is logged without checking this property.
 */
@property(nonatomic) BOOL debug;

/**
 * Enable or disable offline mode. It will be configured in the release process. 
 * Please refer to main page for more info about offlineMode
 */
@property(nonatomic) BOOL offlineMode;

/**
 * Internally it sends http requests to the measurement system.
 * This property sets a timeout for that purpose.
 */
@property(assign) NSTimeInterval timeout;

#ifndef NOT_UNIVERSAL
/**
 * return the instance of Kantar Media Tagging Sensor lib, in id type
 * Initialization of the Kantar Media Tagging Sensor library, with a site name and an application name
 * @warning
 *   The site name and the application name should be provided accrodingly
 */
- (id)initWithSiteAndApplication:(NSString *)theSite application:(NSString *)theApplication;

/**
 * return the instance of Kantar Media Tagging Sensor lib, in id type
 * Initialization of the Kantar Media Tagging Sensor library, with a site name and an application name
 * @warning
 *   The site name and the application name should be provided accrodingly
 */
- (id)initWithSiteAndApplication:(NSString *)theSite application:(NSString *)theApplication withSetting:(NSDictionary *)configurations;
#endif

/**
 * return the instance of Kantar Media Tagging Sensor lib, in id type
 * Initialization of the Kantar Media Tagging Sensor library, with a site name, an application name, a domain and an url schema.
 * Option to manually turn off URLSchemeTrigger is provived via shouldTriggerUrlScheme parameter, Default is YES.
 
 * @warning
 *   This is only allowed for internal test versions. Customer versions will throw an exception if the method is called!
 */
- (id)initWithSiteAndApplication:(NSString *)theSite application:(NSString *)theApplication withDomain:(NSString *)theDomain withUrlScheme:(NSString *)theUrlScheme shouldTriggerUrlScheme:(BOOL)trigger;

/**
 * Commit gathered data to server when application status changes.
 * Initialization of the Kantar Media Tagging Sensor library, with a site name and an application name
 * @throws An exception is thrown when connection problem occurs.
*/
- (void)commit:(NSDictionary *)theDictionary;

/**
 * Returns the encrypted (md5) and truncated mobile identifiers.
 * The MAC ID is stored with the key 'mid'
 * The Vendor ID is stored with the key 'ifv'
 */
- (NSMutableDictionary *) getEncrypedIdentifiers;

@end


/**
 * @mainpage
 
 <div align="center">
 <h2>Kantar Media Tagging Sensor iOS User Manual</h2>
 </div>
 
 <div>
 <h3 style="color: red"> SDK 1.14.0 and higher</h3>
 <p> Starting from SDK version 1.14.0,  we are dropping the usage of IDFA  addressing  the privacy changes that apple introduced in iOS 14.</p>
 <p>Disabling the use of IDFA via constructor is now ignored and it defaults to NO </p>
 </div>
<div>
    <h3 style="color: red">AppStore Submission with iOS 11</h3>
    <p>Please be aware that if you do not include the AdSupport.framework in your app you will have to disable the use of the IDFA via API in the constructor. Otherwise your App may be rejected from the AppStore for legal reasons.</p>
    <p>For more information see the API documentation of the constructor.</p>
</div>
 
<div style="background-color: grey;" >
 <div align="center">
 <h3>Migration to iOS 9 </h3>
 </div>

<h2 style="color:blue;">From version 1.9.0, KMA measuerment libraries will not support iOS6 any more. Kantar Media Tagging library components are all with KMA as prefix from this version, please read more in the API documentation.</h2>
 <h3>URL Scheme</h3>
 <p>Starting on iOS 9, iOS apps will have to declare what URL schemes they would like to be able to check for and open in the configuration files (plist file) of the app as it is submitted to Apple. So if your application is combining with a specific "Panel App" (e.g. Virtualmeter App), please register the URL scheme in your application, the syntax is as following:</p>
 
 <div style="border:1px solid black;">
 <br>&lt;key&gt;LSApplicationQueriesSchemes&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&lt;array&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;string&gt;<span class="marker"><span style="color:#0000CD;">@oldPanelApp</span>&lt;</span>/string&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;string&gt;<span style="color:#0000CD;"><span class="marker">@newPanelApp</span></span>&lt;/string&gt;
 <br>&nbsp;&nbsp; &nbsp;&lt;/array&gt;</div>
 
 
 <h3>App Transport Security (ATS)</h3>
 <p>Starting from iOS 9.0, App Transport Security (ATS) enforces best practices in the secure connections between an app and its back end. Migrating from http to https has to be planed for the more secure communication. However for this moment, if you decide to keep http in Kantar Media Tagging Sensor library, please add following into the project plist file, otherwise please activate ssl on in the library. </p>
 
 <div style="border:1px solid black;">
 &lt;key&gt;NSAppTransportSecurity&lt;/key&gt;
 <br>&lt;dict&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;NSAllowsArbitraryLoads&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;false/&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;NSExceptionDomains&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;dict&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;<span style="color:#0000CD;">@domain</span>&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;dict&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;NSIncludesSubdomains&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;true/&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;NSTemporaryExceptionAllowsInsecureHTTPLoads&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;true/&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;key&gt;NSExceptionRequiresForwardSecrecy&lt;/key&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;false/&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;/dict&gt;
 <br>&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp;&lt;/dict&gt;
 <br>&nbsp;&nbsp; &nbsp;&lt;/dict&gt;</div>
 <p><i><span class="marker"><span style="color:#0000CD;">if</span></span> ATS is not enabled in your application, as NSAllowsArbitraryLoads = true, then you don't need to modify anything.
 </i></p>
 
 <h3>Notification registration for debug purpose</h3>
 
 <p>From KMA tagging iOS version 1.9.1, a notification is inserted into the implementation, so that the customers are able to retrieve the requests sent by the library, see more info #KMA_TAGGING_DEBUGINTERFACE_NOTIFICATION. The notification contains a NSDictionary object, with the keys "Request" and "Statuscode". The following implementation is a sample to fetch the debug info.</p>
 
 <pre><code>
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@ selector(handleApplicationEvent:) name:KMA_TAGGING_DEBUGINTERFACE_NOTIFICATION object:nil];
     &minus; (void) handleApplicationEvent:(NSNotification *)n {
        if(KMA_TAGGING_DEBUGINTERFACE_NOTIFICATION == [n name]) {
            &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;                          have the implementation
        }
    }
 </code></pre>
 <p><i>Please Note: Set the debug mode (KMA_Spring.debug) to false in your live product, otherwise there will be a debug flag in the measurement request.
 </i></p>



 
 <h3>BitCode</h3>
 <p>Bitcode is a new feature on iOS 9, an intermediate representation of a compiled program. Now you have the new Kantar Media Tagging Sensor lib with bitcode enabled, for your application you have the chance to enable or disable the bitcode service.</p>
 </div>


 <h3>Integration into Swift Project</h3>
  <div>
 <p>If the project is a Swift project, a swift bridging header file is required for the integration, more tutorial can be found here:https://kantarmedia.atlassian.net/wiki/spaces/public/pages/159726961/Tutorial+on+how+to+import+KMA+measurement+Objective-C+library+into+Swift+project</p>
 </div>
 
 
 
 
 <h3>Offline Mode</h3>
 <p>iOS-Kantar Media Tagging Sensor lib has feature so called "offlineMode". This mode can be switched on and off by using public API: KMA_Spring.offlineMode.
 If the lib is configured to offlineMode, Kantar Media Tagging Sensor library will hold all requests in a local buffer and send them when the device goes back online. Kantar Media Tagging Sensor lib checks the Internet connection regularly by using the iOS Timer and send the data as soon as possible.
 Please notice:
 -# Old requests will be dropped if too many requests pump into local buffer for the limitation in buffer size. The default buffer size is 500. The data will be stored in a local file, so the lib will not lose the requests even if the application terminates.
 -# Kantar Media Tagging Sensor lib tries to send the requests in a fixed rate, 10 seconds by default, and sends them if device is online.</p>
 
 <h3>WKWebView</h3>
 <div> <p>
 Recently, Apple started warning users that it is going to stop accepting submissions of apps that use UIWebView APIs. Hence, we have removed that.</p>
 <p>Starting from kantarmedia tagging 1.13.0 we are replacing UIWebview with WKWebView  for retreiving Useragent. The integrator must add Webkit framework into the application for the app to run properly.
  </p></div>
 
  <h3>UDID</h3>
 * <table>
 * <tr><th> </th><th>Device ID(did)</th><th>Advertising ID(ai)</th><th>MAC ID(mid)</th><th>ID_For_Vendor(ifv)</th></tr>

 <tr><td>iOS 4</td><td><div align="center">---</div></td><td><div align="center">---</div></td><td>MAC ID(mid)</td><td><div align="center">---</div></td></tr>
 <tr><td>iOS 5</td><td><div align="center">---</div></td><td><div align="center">---</div></td><td>MAC ID(mid)</td><td><div align="center">---</div></td></tr>
 <tr><td>iOS 6</td><td><div align="center">---</div></td><td>Advertising ID(ai)</td><td>MAC ID(mid)</td><td>ID_For_Vendor(ifv)</td></tr>
 <tr><td>iOS 7</td><td><div align="center">---</div></td><td>Advertising ID(ai)</td><td><div align="center">---</div></td><td>ID_For_Vendor(ifv)</td></tr>
 <tr><td>iOS 8</td><td><div align="center">---</div></td><td>Advertising ID(ai)</td><td><div align="center">---</div></td><td>ID_For_Vendor(ifv)</td></tr>
 <tr><td>iOS 9</td><td><div align="center">---</div></td><td>Advertising ID(ai)</td><td><div align="center">---</div></td><td>ID_For_Vendor(ifv)</td></tr>
 * </table>

 <p>Considering that the Apple private policy is changing all the time, Kantar Media Tagging Sensor libs have to adapt different UDIDs for identifying the end user's devices.</p>
 <p>
 -# Since iOS 6, device ID and mac ID are not available anymore,
 -# Switch to Advertising ID and ID_For_Vendor</p>
 
 <p><i>Please attention: Apple will reject all the applications which retrieve advertising ID but with no advertising content provided. So Advertising-Framework is linked as optional in Kantar Media Tagging Sensor libs, If the Advertising ID should be used as udid, please import Advertising-Framework into your projects.</i></p>
 
 <p><i>Please attention: Only initialize Kantar Media Tagging Sensor lib once in your whole project, you only need one KMA_Spring object.</i></p>
 
 
 
 
 <h3>NOTICE</h3>
 <p><i>Please Note: Some components in Kantar Media Tagging Sensor libs are running in background threads. Please keep the initialization and usage of Kantar Media Tagging Sensor libs in your main thread, Kantar Media Tagging Sensor libs will not block your UI display.
</i></p>

 
 
 <div align="center">
 <h2>Package Info</h2>
 </div>
 
 <table>
 <tr><th>File</th><th>Description</th></tr>
 
 <tr><td>spring-appsensor-device.a</td>
 <td>This is the version that has been compiled with ARM support and which is intended for execution on iOS devices</td></tr>
 
 <tr><td>spring-appsensor-simulator.a</td>
 <td>This is the version that has been compiled with x86 support and which is intended for execution on iOS simulator</td></tr>
 
 <tr><td>spring-appsensor-fat.a</td>
 <td>This is a combined version of the two libraries above, which can be executed on both, iOS devices and iOS simulator
 because it contains code for ARM and x86 execution. This file is called "fat" as it is roughly double the size (because it combines both versions)</td></tr>
 
 </table>
 <h4>Note</h4>
 <p><i>If size does not matter for the app, the "fat" version is the carefree option to be used for execution on simulator and real devices.
 </i></p>

 
 <h1>  </h1><h1>  </h1>
 <div align="center">
 <h1><a href="ReleaseNote.html">****RELEASE NOTE****</a></h1>
 </div>



 */
